/*
 * Copyright 2025 cronn GmbH
 * SPDX-License-Identifier: Apache-2.0
 */

package de.eshg.officialmedicalservice.procedure;

import static de.eshg.lib.procedure.domain.factory.SystemProgressEntryFactory.createSystemProgressEntry;
import static de.eshg.officialmedicalservice.concern.ConcernMapper.mapToConcernDto;
import static de.eshg.officialmedicalservice.concern.ConcernMapper.mapToEntity;

import de.eshg.base.centralfile.api.facility.AddFacilityFileStateResponse;
import de.eshg.base.centralfile.api.facility.GetFacilityFileStateResponse;
import de.eshg.base.centralfile.api.facility.GetFacilityFileStatesRequest;
import de.eshg.base.centralfile.api.facility.GetFacilityFileStatesResponse;
import de.eshg.base.centralfile.api.person.AddPersonFileStateResponse;
import de.eshg.base.centralfile.api.person.GetPersonFileStateResponse;
import de.eshg.base.centralfile.api.person.GetPersonFileStatesRequest;
import de.eshg.base.centralfile.api.person.GetPersonFileStatesResponse;
import de.eshg.base.user.api.UserDto;
import de.eshg.lib.auditlog.AuditLogger;
import de.eshg.lib.procedure.domain.model.FacilityType;
import de.eshg.lib.procedure.domain.model.ProcedureStatus;
import de.eshg.lib.procedure.domain.model.RelatedPerson;
import de.eshg.lib.procedure.domain.model.TriggerType;
import de.eshg.lib.procedure.mapping.ProcedureMapper;
import de.eshg.officialmedicalservice.appointment.OmsAppointmentMapper;
import de.eshg.officialmedicalservice.concern.ConcernMapper;
import de.eshg.officialmedicalservice.facility.FacilityClient;
import de.eshg.officialmedicalservice.facility.FacilityMapper;
import de.eshg.officialmedicalservice.person.PersonClient;
import de.eshg.officialmedicalservice.person.PersonMapper;
import de.eshg.officialmedicalservice.procedure.api.AffectedPersonDto;
import de.eshg.officialmedicalservice.procedure.api.EmployeeOmsProcedureDetailsDto;
import de.eshg.officialmedicalservice.procedure.api.EmployeeOmsProcedureHeaderDto;
import de.eshg.officialmedicalservice.procedure.api.EmployeeOmsProcedureOverviewDto;
import de.eshg.officialmedicalservice.procedure.api.EmployeeOmsProcedurePaginationAndSortParameters;
import de.eshg.officialmedicalservice.procedure.api.EmployeePagedOmsProcedures;
import de.eshg.officialmedicalservice.procedure.api.FacilityDto;
import de.eshg.officialmedicalservice.procedure.api.PatchAffectedPersonRequest;
import de.eshg.officialmedicalservice.procedure.api.PatchConcernRequest;
import de.eshg.officialmedicalservice.procedure.api.PatchEmployeeOmsProcedureFacilityRequest;
import de.eshg.officialmedicalservice.procedure.api.PatchEmployeeOmsProcedurePhysicianRequest;
import de.eshg.officialmedicalservice.procedure.api.PostEmployeeOmsProcedureFacilityRequest;
import de.eshg.officialmedicalservice.procedure.api.PostEmployeeOmsProcedureRequest;
import de.eshg.officialmedicalservice.procedure.api.SyncAffectedPersonRequest;
import de.eshg.officialmedicalservice.procedure.api.SyncFacilityRequest;
import de.eshg.officialmedicalservice.procedure.persistence.entity.Concern;
import de.eshg.officialmedicalservice.procedure.persistence.entity.Facility;
import de.eshg.officialmedicalservice.procedure.persistence.entity.OmsProcedure;
import de.eshg.officialmedicalservice.procedure.persistence.entity.OmsProcedureRepository;
import de.eshg.officialmedicalservice.procedure.persistence.entity.Person;
import de.eshg.officialmedicalservice.user.UserClient;
import de.eshg.rest.service.error.BadRequestException;
import de.eshg.rest.service.error.NotFoundException;
import de.eshg.rest.service.security.CurrentUserHelper;
import de.eshg.validation.ValidationUtil;
import java.time.Clock;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
public class EmployeeOmsProcedureService {
  private final OmsProcedureRepository omsProcedureRepository;
  private final OmsProcedureOverviewMapper omsProcedureOverviewMapper;
  private final OmsAppointmentMapper omsAppointmentMapper;
  private final PersonClient personClient;
  private final FacilityClient facilityClient;
  private final UserClient userClient;
  private final Clock clock;
  private final AuditLogger auditLogger;
  private final ProgressEntryService progressEntryService;

  public EmployeeOmsProcedureService(
      OmsProcedureRepository omsProcedureRepository,
      OmsProcedureOverviewMapper omsProcedureOverviewMapper,
      OmsAppointmentMapper omsAppointmentMapper,
      PersonClient personClient,
      FacilityClient facilityClient,
      Clock clock,
      AuditLogger auditLogger,
      UserClient userClient,
      ProgressEntryService progressEntryService) {
    this.omsProcedureRepository = omsProcedureRepository;
    this.omsProcedureOverviewMapper = omsProcedureOverviewMapper;
    this.omsAppointmentMapper = omsAppointmentMapper;
    this.personClient = personClient;
    this.facilityClient = facilityClient;
    this.clock = clock;
    this.auditLogger = auditLogger;
    this.userClient = userClient;
    this.progressEntryService = progressEntryService;
  }

  @Transactional
  public UUID createEmployeeProcedure(PostEmployeeOmsProcedureRequest request) {
    AddPersonFileStateResponse affectedPersonBaseResponse =
        personClient.addPersonFileState(
            PersonMapper.mapToAddPersonFileStateRequest(request.affectedPerson()));

    OmsProcedure procedure =
        omsProcedureOverviewMapper.toDomainType(
            request, CurrentUserHelper.getCurrentUserId(), affectedPersonBaseResponse, null);

    omsProcedureRepository.save(procedure);

    return procedure.getExternalId();
  }

  @Transactional(readOnly = true)
  public EmployeeOmsProcedureHeaderDto getEmployeeProcedureHeader(UUID externalId) {
    OmsProcedureAndAffectedPerson omsProcedureAndAffectedPerson =
        getOmsProcedureAndAffectedPerson(externalId);

    return new EmployeeOmsProcedureHeaderDto(
        omsProcedureAndAffectedPerson.omsProcedure.getExternalId(),
        ProcedureMapper.toInterfaceType(
            omsProcedureAndAffectedPerson.omsProcedure.getProcedureStatus()),
        omsProcedureAndAffectedPerson.affectedPerson.firstName(),
        omsProcedureAndAffectedPerson.affectedPerson.lastName(),
        omsProcedureAndAffectedPerson.affectedPerson.dateOfBirth());
  }

  @Transactional(readOnly = true)
  public EmployeeOmsProcedureDetailsDto getEmployeeProcedureDetails(UUID externalId) {
    OmsProcedureAndAffectedPerson omsProcedureAndAffectedPerson =
        getOmsProcedureAndAffectedPerson(externalId);

    FacilityDto facility = null;
    Optional<Facility> optionalFacility = omsProcedureAndAffectedPerson.omsProcedure.getFacility();
    if (optionalFacility.isPresent()) {
      Facility facilityFromCentralFile = optionalFacility.get();
      GetFacilityFileStateResponse facilityFileState =
          facilityClient.getFacilityFileState(facilityFromCentralFile.getCentralFileStateId());
      facility =
          FacilityMapper.mapToFacilityDto(facilityFileState, facilityFromCentralFile.getVersion());
    }

    UUID physicianId = omsProcedureAndAffectedPerson.omsProcedure.getPhysicianId();
    Optional<UserDto> physician = userClient.retrievePhysician(physicianId);

    return new EmployeeOmsProcedureDetailsDto(
        omsProcedureAndAffectedPerson.omsProcedure.getExternalId(),
        ProcedureMapper.toInterfaceType(
            omsProcedureAndAffectedPerson.omsProcedure.getProcedureStatus()),
        omsProcedureAndAffectedPerson.affectedPerson,
        facility,
        mapToConcernDto(omsProcedureAndAffectedPerson.omsProcedure.getConcern()),
        physician.orElse(null),
        omsAppointmentMapper.toInterfaceType(
            omsProcedureAndAffectedPerson.omsProcedure.getAppointments()));
  }

  @Transactional(readOnly = true)
  public EmployeePagedOmsProcedures getEmployeeProceduresOverview(
      EmployeeOmsProcedurePaginationAndSortParameters paginationAndSortParameters) {

    Page<OmsProcedure> omsProcedures =
        omsProcedureRepository.findAll(
            new EmployeeOmsProcedureSpecification(paginationAndSortParameters),
            EmployeeOmsProcedureSpecification.toPageSpec(paginationAndSortParameters));

    Map<UUID, GetPersonFileStateResponse> personMap = getPersonMap(omsProcedures.getContent());
    Map<UUID, GetFacilityFileStateResponse> facilityMap =
        getFacilityMap(omsProcedures.getContent());
    Map<UUID, UserDto> physicianMap = userClient.getPhysiciansMap();

    List<EmployeeOmsProcedureOverviewDto> omsProcedureOverviewDtos =
        omsProcedures.getContent().stream()
            .map(
                omsProcedure ->
                    omsProcedureOverviewMapper.toInterfaceType(
                        omsProcedure,
                        getPersonForOmsProcedure(omsProcedure, personMap),
                        getFacilityForOmsProcedure(omsProcedure, facilityMap),
                        getPhysicianForOmsProcedure(omsProcedure, physicianMap)))
            .toList();

    return new EmployeePagedOmsProcedures(
        omsProcedureOverviewDtos, omsProcedures.getTotalElements());
  }

  @Transactional
  public void abortDraftProcedure(UUID externalId) {
    OmsProcedure omsProcedure = loadOmsProcedure(externalId);

    if (omsProcedure.getProcedureStatus() != ProcedureStatus.DRAFT) {
      throw new BadRequestException("Procedure is not in DRAFT status");
    }

    omsProcedureRepository.deleteById(omsProcedure.getId());
  }

  @Transactional
  public void acceptDraftProcedure(UUID externalId) {
    OmsProcedure omsProcedure = loadOmsProcedureForUpdate(externalId);

    if (omsProcedure.getProcedureStatus() != ProcedureStatus.DRAFT) {
      throw new BadRequestException("Procedure is not in DRAFT status");
    }
    requireFacility(omsProcedure);
    requireConcern(omsProcedure);

    omsProcedure.updateProcedureStatus(ProcedureStatus.OPEN, clock, auditLogger);
    omsProcedure.addProgressEntry(
        createSystemProgressEntry(
            OmsProgressEntryType.PROCEDURE_STARTED.name(), TriggerType.EMPLOYEE));
  }

  @Transactional
  public void closeOpenProcedure(UUID externalID) {
    OmsProcedure omsProcedure = loadOmsProcedureForUpdate(externalID);

    if (omsProcedure.getProcedureStatus() == ProcedureStatus.CLOSED) {
      throw new BadRequestException("Procedure is already in CLOSED status");
    }
    if (omsProcedure.getProcedureStatus() != ProcedureStatus.OPEN) {
      throw new BadRequestException("Procedure is not in OPEN status");
    }

    // ToDo require all appointments closed

    omsProcedure.updateProcedureStatus(ProcedureStatus.CLOSED, clock, auditLogger);
  }

  @Transactional
  public void updateAffectedPerson(
      UUID externalId, PatchAffectedPersonRequest patchAffectedPersonRequest) {
    OmsProcedure omsProcedure = loadOmsProcedureForUpdate(externalId);

    if (omsProcedure.getProcedureStatus() == ProcedureStatus.CLOSED) {
      throw new BadRequestException("Affected person can not be edited in CLOSED status");
    }

    Person person = omsProcedure.findAffectedPerson();

    ValidationUtil.validateVersion(patchAffectedPersonRequest.affectedPerson().version(), person);

    AddPersonFileStateResponse baseResponse;
    try {
      baseResponse =
          personClient.updatePersonFileStateAndReference(
              person.getCentralFileStateId(),
              PersonMapper.mapToUpdatePersonRequest(patchAffectedPersonRequest.affectedPerson()));
    } catch (BadRequestException e) {
      if (!"Matching reference Person already exists".equals(e.getMessage())) {
        throw e;
      }
      baseResponse =
          personClient.addPersonFileState(
              PersonMapper.mapToAddPersonFileStateRequest(
                  patchAffectedPersonRequest.affectedPerson()));
    }

    person.setCentralFileStateId(baseResponse.id());
  }

  @Transactional
  public void syncAffectedPerson(UUID externalId, SyncAffectedPersonRequest request) {
    OmsProcedure procedure = loadOmsProcedureForUpdate(externalId);

    if (procedure.getProcedureStatus() == ProcedureStatus.CLOSED) {
      throw new BadRequestException("Affected person can not be synced in CLOSED status");
    }

    Person person = procedure.getRelatedPersons().getFirst();
    UUID updateFileStateId =
        personClient.syncAffectedPerson(person.getCentralFileStateId(), request.referenceVersion());
    person.setCentralFileStateId(updateFileStateId);

    progressEntryService.createProgressEntryForSyncAffectedPerson(procedure);
  }

  @Transactional
  public UUID addFacility(UUID externalId, PostEmployeeOmsProcedureFacilityRequest request) {
    OmsProcedure procedure = loadOmsProcedure(externalId);

    if (procedure.getProcedureStatus() != ProcedureStatus.DRAFT) {
      throw new BadRequestException("Facility can only be added in DRAFT status");
    }
    if (procedure.getFacility().isPresent()) {
      throw new BadRequestException("Procedure already has a facility");
    }
    AddFacilityFileStateResponse facilityFileState =
        facilityClient.addFacilityFileState(
            FacilityMapper.mapToAddFacilityFileStateRequest(request.facility()));

    Facility facility = new Facility();
    facility.setCentralFileStateId(facilityFileState.id());
    facility.setFacilityType(FacilityType.OTHER);
    procedure.addRelatedFacility(facility);

    return facilityFileState.id();
  }

  @Transactional
  public void updateFacility(UUID externalId, PatchEmployeeOmsProcedureFacilityRequest request) {
    OmsProcedure procedure = loadOmsProcedureForUpdate(externalId);

    if (procedure.getProcedureStatus() == ProcedureStatus.CLOSED) {
      throw new BadRequestException("Facility can not be edited in CLOSED status");
    }
    if (procedure.getProcedureStatus() != ProcedureStatus.DRAFT) {
      throw new BadRequestException("Facility can only be synced in DRAFT status");
    }

    Optional<Facility> optionalFacility = procedure.getFacility();
    if (optionalFacility.isEmpty()) {
      throw new BadRequestException("Procedure doesn't have a facility");
    }
    Facility facility = optionalFacility.get();
    ValidationUtil.validateVersion(request.updatedFacility().version(), facility);

    AddFacilityFileStateResponse baseResponse;
    try {
      baseResponse =
          facilityClient.updateFacilityFileStateAndReference(
              facility.getCentralFileStateId(),
              FacilityMapper.mapToPutFacilityRequest(request.updatedFacility()));
    } catch (BadRequestException e) {
      if (!"Matching reference facility already exists".equals(e.getMessage())) {
        throw e;
      }
      baseResponse =
          facilityClient.addFacilityFileState(
              FacilityMapper.mapToAddFacilityFileStateRequest(request.updatedFacility()));
    }

    facility.setCentralFileStateId(baseResponse.id());
  }

  @Transactional
  public void syncFacilityData(UUID externalId, SyncFacilityRequest request) {
    OmsProcedure procedure = loadOmsProcedureForUpdate(externalId);

    if (procedure.getProcedureStatus() == ProcedureStatus.CLOSED) {
      throw new BadRequestException("Facility can not be synced in CLOSED status");
    }
    if (procedure.getProcedureStatus() != ProcedureStatus.DRAFT) {
      throw new BadRequestException("Facility can only be synced in DRAFT status");
    }

    Optional<Facility> optionalFacility = procedure.getFacility();
    if (optionalFacility.isEmpty()) {
      throw new BadRequestException("Procedure doesn't have a facility");
    }
    Facility facility = optionalFacility.get();

    UUID updatedFileStateId =
        facilityClient.syncFacility(facility.getCentralFileStateId(), request.referenceVersion());
    facility.setCentralFileStateId(updatedFileStateId);

    progressEntryService.createProgressEntryForSyncFacility(procedure);
  }

  @Transactional
  public UUID modifyPhysician(UUID externalId, PatchEmployeeOmsProcedurePhysicianRequest request) {
    OmsProcedure procedure = loadOmsProcedure(externalId);

    if (procedure.getProcedureStatus() == ProcedureStatus.CLOSED) {
      throw new BadRequestException("A physician can not be set in CLOSED status");
    }

    UUID newPhysicianId = request.physicianId();
    UserDto newPhysician =
        userClient.validateUser(newPhysicianId, userClient.getTechnicalGroupPhysicians());

    procedure.setPhysicianId(newPhysicianId);
    progressEntryService.createProgressEntryForModifiedPhysician(procedure, newPhysician);
    return newPhysicianId;
  }

  @Transactional
  public void updateOmsProcedureConcern(UUID externalId, PatchConcernRequest request) {
    OmsProcedure omsProcedure = loadOmsProcedureForUpdate(externalId);

    if (omsProcedure.getProcedureStatus() == ProcedureStatus.CLOSED) {
      throw new BadRequestException("Concern can not be edited in CLOSED status");
    }
    if (omsProcedure.getProcedureStatus() != ProcedureStatus.DRAFT) {
      throw new BadRequestException("Procedure is not in DRAFT status");
    }

    Concern existingConcern = omsProcedure.getConcern();

    if (existingConcern != null) {
      ValidationUtil.validateVersion(request.concern().version(), existingConcern);
      ConcernMapper.mapOntoExistingEntity(request.concern(), existingConcern);
    } else {
      omsProcedure.setConcern(mapToEntity(request.concern()));
    }
  }

  private OmsProcedure loadOmsProcedure(UUID externalId) {
    return omsProcedureRepository
        .findByExternalId(externalId)
        .orElseThrow(() -> new NotFoundException("Procedure not found"));
  }

  private OmsProcedure loadOmsProcedureForUpdate(UUID externalId) {
    return omsProcedureRepository
        .findByExternalIdForUpdate(externalId)
        .orElseThrow(() -> new NotFoundException("Procedure not found"));
  }

  private OmsProcedureAndAffectedPerson getOmsProcedureAndAffectedPerson(UUID externalId) {
    OmsProcedure omsProcedure = loadOmsProcedure(externalId);
    Person person = omsProcedure.findAffectedPerson();

    GetPersonFileStateResponse personFileStateResponse =
        personClient.getPersonFileState(person.getCentralFileStateId());

    AffectedPersonDto affectedPerson =
        PersonMapper.mapToAffectedPersonDto(personFileStateResponse, person.getVersion());

    return new OmsProcedureAndAffectedPerson(omsProcedure, affectedPerson);
  }

  private Map<UUID, GetPersonFileStateResponse> getPersonMap(List<OmsProcedure> omsProcedures) {
    List<UUID> centralFileStateIds =
        omsProcedures.stream()
            .map(OmsProcedure::findAffectedPerson)
            .map(RelatedPerson::getCentralFileStateId)
            .distinct()
            .toList();

    if (centralFileStateIds.isEmpty()) {
      return Collections.emptyMap();
    }

    GetPersonFileStatesResponse personFileStatesResponse =
        personClient.getPersonFileStates(
            new GetPersonFileStatesRequest(
                omsProcedures.stream()
                    .map(OmsProcedure::findAffectedPerson)
                    .filter(Objects::nonNull)
                    .map(RelatedPerson::getCentralFileStateId)
                    .distinct()
                    .toList()));

    return personFileStatesResponse.personFileStates().stream()
        .collect(Collectors.toMap(GetPersonFileStateResponse::id, person -> person));
  }

  private Map<UUID, GetFacilityFileStateResponse> getFacilityMap(List<OmsProcedure> omsProcedures) {
    List<UUID> centralFileStateIds =
        omsProcedures.stream()
            .map(OmsProcedure::getFacility)
            .flatMap(Optional::stream)
            .map(Facility::getCentralFileStateId)
            .distinct()
            .toList();

    if (centralFileStateIds.isEmpty()) {
      return Collections.emptyMap();
    }

    GetFacilityFileStatesResponse facilityFileStatesResponse =
        facilityClient.getFacilityFileStates(new GetFacilityFileStatesRequest(centralFileStateIds));

    return facilityFileStatesResponse.facilityFileStates().stream()
        .collect(Collectors.toMap(GetFacilityFileStateResponse::id, facility -> facility));
  }

  private GetPersonFileStateResponse getPersonForOmsProcedure(
      OmsProcedure omsProcedure, Map<UUID, GetPersonFileStateResponse> personMap) {
    if (omsProcedure.findAffectedPerson() == null) {
      return null;
    }
    return personMap.get(omsProcedure.findAffectedPerson().getCentralFileStateId());
  }

  private GetFacilityFileStateResponse getFacilityForOmsProcedure(
      OmsProcedure omsProcedure, Map<UUID, GetFacilityFileStateResponse> facilityMap) {
    return omsProcedure
        .getFacility()
        .map(facility -> facilityMap.get(facility.getCentralFileStateId()))
        .orElse(null);
  }

  private void requireFacility(OmsProcedure omsProcedure) {
    if (omsProcedure.getFacility().isEmpty()) {
      throw new BadRequestException("Procedure is missing a facility");
    }
  }

  private void requireConcern(OmsProcedure omsProcedure) {
    if (omsProcedure.getConcern() == null) {
      throw new BadRequestException("Procedure is missing a concern");
    }
  }

  private UserDto getPhysicianForOmsProcedure(
      OmsProcedure omsProcedure, Map<UUID, UserDto> physicianMap) {
    return Optional.ofNullable(omsProcedure.getPhysicianId()).map(physicianMap::get).orElse(null);
  }

  private record OmsProcedureAndAffectedPerson(
      OmsProcedure omsProcedure, AffectedPersonDto affectedPerson) {}
}
