/**
 * Copyright 2025 cronn GmbH
 * SPDX-License-Identifier: AGPL-3.0-only
 */

"use client";

import { Box } from "@mui/joy";
import { startTransition, useState } from "react";

import { useExportReportData } from "@/lib/businessModules/statistics/api/downloads/useExportReportData";
import { ReportDataType } from "@/lib/businessModules/statistics/api/models/evaluationReports";
import { ReportOverviewTableRow } from "@/lib/businessModules/statistics/api/models/reportsOverviewTypes";
import { useGetReportsOverview } from "@/lib/businessModules/statistics/api/queries/useGetReportsOverview";
import { createFilterDefinitions } from "@/lib/businessModules/statistics/components/reports/filterDefinitions";
import { useDeleteWithConfirmation } from "@/lib/businessModules/statistics/components/reports/useDeleteWithConfirmation";
import { useDataExportGuard } from "@/lib/businessModules/statistics/components/shared/hooks/useDataExportGuard";
import { useStatisticsRoleChecks } from "@/lib/businessModules/statistics/permissions/useStatisticsRoleChecks";
import { routes } from "@/lib/businessModules/statistics/shared/routes";
import { NoSearchResults } from "@/lib/shared/components/NoSearchResult";
import { ButtonBar } from "@/lib/shared/components/buttons/ButtonBar";
import { FilterButton } from "@/lib/shared/components/buttons/FilterButton";
import { FilterSettings } from "@/lib/shared/components/filterSettings/FilterSettings";
import { FilterSettingsSheet } from "@/lib/shared/components/filterSettings/FilterSettingsSheet";
import { FilterValue } from "@/lib/shared/components/filterSettings/models/FilterValue";
import { useFilterSettings } from "@/lib/shared/components/filterSettings/useFilterSettings";
import { Pagination } from "@/lib/shared/components/pagination/Pagination";
import { DataTable } from "@/lib/shared/components/table/DataTable";
import { TablePage } from "@/lib/shared/components/table/TablePage";
import { TableSheet } from "@/lib/shared/components/table/TableSheet";
import { usePagination } from "@/lib/shared/hooks/table/usePagination";
import { useCopy } from "@/lib/shared/hooks/useCopy";

import { getReportsOverviewColumns } from "./columns";

export function ReportsOverview() {
  const copy = useCopy();

  const { deleteReportWithConfirmation, deleteReportSeriesWithConfirmation } =
    useDeleteWithConfirmation();
  const { download: exportData } = useExportReportData();
  const userPermissions = useStatisticsRoleChecks();

  const { resetPageNumber, page, pageSize, getPaginationProps } =
    usePagination();

  const [filterValues, setFilterValues] = useState<FilterValue[]>([]);

  const { dataSources, reportsOverview } = useGetReportsOverview(
    {
      page,
      pageSize,
      sortDirection: undefined,
      sortKey: undefined,
    },
    filterValues,
  );
  const dataExportGuard = useDataExportGuard();

  const filterSettings = useFilterSettings({
    definitions: createFilterDefinitions(dataSources),
    onValuesSubmit: (filterValues) => {
      startTransition(() => {
        setFilterValues(filterValues);
        resetPageNumber();
      });
    },
    showSearch: false,
  });

  const paginationProps = getPaginationProps({
    totalCount: reportsOverview.totalNumberOfElements,
  });

  function getSubRows(item: ReportOverviewTableRow) {
    return item.type === "SERIES" ? item.subRows : undefined;
  }

  return (
    <TablePage
      data-testid="evaluations-reports-overview-table"
      fullHeight
      controls={
        <ButtonBar
          left={<FilterButton {...filterSettings.filterButtonProps} />}
        />
      }
      filterSettings={
        filterSettings.filterSettingsVisible && (
          <FilterSettingsSheet {...filterSettings.filterSettingsSheetProps}>
            <FilterSettings {...filterSettings.filterSettingsProps} />
          </FilterSettingsSheet>
        )
      }
    >
      <TableSheet footer={<Pagination {...paginationProps} />}>
        <DataTable
          striped={false}
          wrapContent
          columns={getReportsOverviewColumns(
            copy,
            deleteReportWithConfirmation,
            deleteReportSeriesWithConfirmation,
            async (item) =>
              dataExportGuard(item.dataSensitivity, () =>
                exportData(
                  { reportId: item.reportId },
                  { tooMuchDataForExport: item.tooMuchDataForExport },
                ),
              ),
            userPermissions.canWrite(),
            userPermissions.canDelete,
          )}
          data={reportsOverview.reports}
          noDataComponent={() => (
            <Box flex={1} alignContent="center">
              <NoSearchResults info="Keine Reports vorhanden" />
            </Box>
          )}
          rowNavigation={{
            route: (row) =>
              // For Reports overview, only completed reports are shown
              row.original.type !== ReportDataType.Series
                ? routes.reports.details(row.original.reportId).index
                : undefined,
            focusColumnAccessorKey: "name",
          }}
          getSubRows={getSubRows}
        />
      </TableSheet>
    </TablePage>
  );
}
