/**
 * Copyright 2025 cronn GmbH
 * SPDX-License-Identifier: Apache-2.0
 */

import { ApiBaseFeature } from "@eshg/base-api";
import {
  ApiBusinessModule,
  ApiGdprDownloadPackageInfo,
  ApiGetGdprDownloadPackagesInfoResponse,
  ApiGetGdprNotificationBannerResponse,
  GdprValidationTaskApiInterface,
  GetAllGdprValidationTasksRequest,
} from "@eshg/employee-portal-api/businessProcedures";
import { unwrapRawResponse } from "@eshg/lib-portal/api/unwrapRawResponse";
import { PortalErrorCode } from "@eshg/lib-portal/errorHandling/PortalErrorCode";
import { resolveError } from "@eshg/lib-portal/errorHandling/errorResolvers";
import { queryOptions, useSuspenseQueries } from "@tanstack/react-query";
import assert from "assert";
import { isDefined } from "remeda";

import { useGdprProcedureApi } from "@/lib/baseModule/api/clients";
import { gdprValidationTaskApiQueryKey } from "@/lib/baseModule/api/queries/apiQueryKey";
import { useServerConfig } from "@/lib/baseModule/api/queries/config";
import { useIsNewFeatureEnabled as useIsNewBaseFeatureEnabled } from "@/lib/baseModule/api/queries/feature";
import { useGdprValidationTaskApi } from "@/lib/shared/api/clients";

const businessModules = Object.freeze(Object.values(ApiBusinessModule));

export function useGetGdprValidationBannerQuery(
  businessModule: ApiBusinessModule,
) {
  const taskApi = useGdprValidationTaskApi(businessModule);
  const isGdprFeatureEnabled = useIsNewBaseFeatureEnabled(ApiBaseFeature.Gdpr);
  return queryOptions({
    queryKey: gdprValidationTaskApiQueryKey([
      businessModule,
      "getGdprNotificationBanner",
      isGdprFeatureEnabled,
    ]),
    queryFn: async (): Promise<ApiGetGdprNotificationBannerResponse> => {
      if (isGdprFeatureEnabled) {
        return await taskApi.getGdprNotificationBanner();
      } else {
        return {
          openValidationTasksCount: 0,
        };
      }
    },
  });
}

export function getGdprValidationTaskDetailsQuery(
  taskApi: GdprValidationTaskApiInterface,
  businessModule: ApiBusinessModule,
  id: string,
) {
  return queryOptions({
    queryKey: gdprValidationTaskApiQueryKey([
      businessModule,
      "getGdprValidationTaskDetails",
      id,
    ]),
    queryFn: () => taskApi.getGdprValidationTaskDetails(id),
  });
}

export function getGdprValidationTasksQuery(
  taskApi: GdprValidationTaskApiInterface,
  businessModule: ApiBusinessModule,
  request: GetAllGdprValidationTasksRequest,
) {
  return queryOptions({
    queryKey: gdprValidationTaskApiQueryKey([
      businessModule,
      "getGdprValidationTasks",
      request,
    ]),
    queryFn: () =>
      taskApi.getAllGdprValidationTasksRaw(request).then(unwrapRawResponse),
  });
}

interface DownloadPackagesQueryResponse {
  businessModule: ApiBusinessModule;
  downloadPackages: ApiGdprDownloadPackageInfo[];
}

function getGdprDownloadPackagesInfoQuery(
  taskApi: GdprValidationTaskApiInterface,
  businessModule: ApiBusinessModule,
  gdprProcedureId: string,
  enabled: boolean,
) {
  return queryOptions({
    queryKey: gdprValidationTaskApiQueryKey([
      businessModule,
      "getGdprDownloadPackagesInfo",
      gdprProcedureId,
      enabled,
    ]),
    queryFn: async (): Promise<
      "disabled" | ApiGetGdprDownloadPackagesInfoResponse
    > => {
      if (enabled) {
        try {
          return await taskApi.getGdprDownloadPackagesInfo(gdprProcedureId);
        } catch (e: unknown) {
          const resolved = resolveError(e);
          if (resolved.errorCode === PortalErrorCode.NotFound) {
            return "disabled";
          }
          throw e;
        }
      } else {
        return "disabled";
      }
    },
    select: (data): DownloadPackagesQueryResponse => {
      if (data === "disabled") {
        return {
          businessModule,
          downloadPackages: [],
        };
      } else {
        return {
          businessModule,
          downloadPackages: data.downloadPackages,
        };
      }
    },
  });
}

export function useGetGdprDownloadPackagesInfo(
  gdprProcedureId: string,
  enabled: boolean,
) {
  const { data: config } = useServerConfig();
  const queries = businessModules.map((module) => {
    // Using hooks in a loop is allowed here, since the businessModules array is constant.
    // eslint-disable-next-line react-hooks/rules-of-hooks
    const gdprValidationTaskApi = useGdprValidationTaskApi(module);
    return getGdprDownloadPackagesInfoQuery(
      gdprValidationTaskApi,
      module,
      gdprProcedureId,
      enabled && config.activeModules.includes(module),
    );
  });

  return useSuspenseQueries({
    queries,
  });
}

export function useDownloadPackageFileByModule() {
  const moduleApiHooks = businessModules.map((module) => ({
    module,
    // Using hooks in a loop is allowed here, since the businessModules array is constant.
    // eslint-disable-next-line react-hooks/rules-of-hooks
    api: useGdprValidationTaskApi(module),
  }));

  function downloadPackage(
    businessModule: ApiBusinessModule,
    gdprProcedureId: string,
    downloadId: string,
  ) {
    const resolved = moduleApiHooks.find(
      ({ module }) => module === businessModule,
    )!;
    assert(
      isDefined(resolved),
      `Module mapping for API should be defined for business module ${businessModule}`,
    );
    return resolved.api.getGdprDownloadPackageRaw({
      gdprProcedureId,
      downloadId,
    });
  }

  return downloadPackage;
}

export function useDownloadBaseModulePackage() {
  const gdprApi = useGdprProcedureApi();
  return (gdprProcedureId: string) =>
    gdprApi.getCentralFileDownloadPackageRaw({ id: gdprProcedureId });
}
